/**************************************************************************************************
					LCD-Routinen
			-------------------


			 4-Bit Interface/Modus
 			=====================

------------------------------------------------------------
PD7		PD6		PD5 	PD4		PD3	PD2 	PD1 	PD0

--       --       RS    E        DB7   DB6   DB5   DB4
--       --                      DB3   DB2   DB1   DB0
                  1=Befehl	Enable
                  0=Daten

File: 	lcd_routines.c
Version:1.0

History:
    -(30.09.2007)Funktionsfhiges Kode: Einfaches "Hallo World!" Anzeige
    -(15.12.2008)xxxxx

**************************************************************************************************/

/* Pinbelegung ber defines einstellbar */

#include <avr/io.h>
#include "lcd_routines.h"
#include <util/delay.h>


/* LCD Befehle */
#define CLEAR_DISPLAY 0x01
#define CURSOR_HOME   0x02

/* Pinbelegung fr das LCD */
#define LCD_PORT      PORTD
#define LCD_DDR       DDRD
#define LCD_RS        5
#define LCD_EN        4



/* Initialisierung:
   Muss ganz am Anfang des Programms aufgerufen werden. */
void lcd_init(void)
{
   LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) | (1<<LCD_EN);   /* Port auf Ausgang schalten */

   /* muss 3mal hintereinander gesendet werden zur Initialisierung */

    _delay_ms(41);
   LCD_PORT &= 0xF0;
   LCD_PORT |= 0x03;
   LCD_PORT &= ~(1<<LCD_RS);      /* RS auf 0 */
   lcd_enable();

    _delay_ms(41);
   lcd_enable();

    _delay_ms(41);
   lcd_enable();

   /* 4 Bit Modus aktivieren */
   LCD_PORT &= 0xF0;
   LCD_PORT |= 0x02;
   lcd_enable();

   /* 4Bit / 2 Zeilen / 5x7  (Funktion Set)*/
    lcd_command(0x28);

   /* Display ein / Cursor aus / kein Blinken */
   lcd_command(0x0C);

   /* inkrement / kein Scrollen (Entry Mode Set)*/
   lcd_command(0x06);

    lcd_clear();
}



/* sendet ein Datenbyte an das LCD */
void lcd_data(unsigned char temp1)
{
  unsigned char temp2 = temp1;		/* temp1 retten */

  LCD_PORT |= (1<<LCD_RS); /* RS auf 1 setzen */

  temp1 = temp1 >> 4;		/* Oberes Nibble holen */
  temp1 = temp1 & 0x0F;		/* Lsche obere nibble */
  LCD_PORT &= 0xF0;			/* Untere nibble maskieren */
  LCD_PORT |= temp1;       /* Am Port ausgeben */
  lcd_enable();


  temp2 = temp2 & 0x0F;
  LCD_PORT &= 0xF0;
  LCD_PORT |= temp2;       /* setzen */
  lcd_enable();


  _delay_us(42);
}

/* sendet einen Befehl an das LCD */
void lcd_command(unsigned char temp1)
{
  unsigned char temp2 = temp1;

  LCD_PORT &= ~(1<<LCD_RS);       /* RS auf 0 setzen */

  temp1 = temp1 >> 4;              /* Oberes Nibble holen */
  temp1 = temp1 & 0x0F;            /* maskieren */
  LCD_PORT &= 0xF0;
  LCD_PORT |= temp1;               /* setzen */
  lcd_enable();

  temp2 = temp2 & 0x0F;            /* Unteres Nibble maskieren */
  LCD_PORT &= 0xF0;
  LCD_PORT |= temp2;               /* setzen */
  lcd_enable();


  _delay_us(42);
}

/* erzeugt den Enable-Puls */
void lcd_enable(void)
{
   LCD_PORT |= (1<<LCD_EN);
    _delay_us(1);                   /* kurze Pause */
   LCD_PORT &= ~(1<<LCD_EN); 		/* Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern */
}

/* Sendet den Befehl zur Lschung des Displays */
void lcd_clear(void)
{
   lcd_command(CLEAR_DISPLAY);
   _delay_ms(5);
}

/* Sendet den Befehl: Cursor Home */
void lcd_home(void)
{
   lcd_command(CURSOR_HOME);
   _delay_ms(5);
}

/* setzt den Cursor in Zeile y (1..4) Spalte x (0..15) */
// setzt den Cursor in Zeile y (1..4) Spalte x (0..15)

void set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp;

  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    case 3: tmp=0x80+0x14+x; break;    // 3. Zeile
    case 4: tmp=0x80+0x54+x; break;    // 4. Zeile
    default: return;                   // fr den Fall einer falschen Zeile
  }
  lcd_command(tmp);
}


/* Schreibt einen String auf das LCD */
void lcd_string(char *data)
{
    while(*data) {
        lcd_data(*data);
        data++;
    }
}
